package lc.esp.sdk.telemetry;

import lc.mecha.json.JSONArray;
import lc.mecha.json.JSONObject;

import java.time.Instant;
import java.util.HashSet;
import java.util.Set;

/**
 * A telemetry frame contains zero or more symbols which are grouped together into a single "reading".
 * <p>
 * If symbols happen to be inter-related, for example, a X-Y-Z positional vector, then they are said to be of the
 * same reading if they are within the same frame. The frame constitutes a particular instant of time.
 *
 * @author Alex Leigh
 * @since mk18 (GIPSY DANGER)
 */
public class TelemetryFrame {
    private final HashSet<TelemetrySymbol> symbols = new HashSet<>();
    private final Instant time;

    public TelemetryFrame(Instant time) {
        this.time = time;
    }

    public Instant getTime() {
        return time;
    }

    public JSONObject toJson() {
        JSONObject frameJson = new JSONObject();
        frameJson.put("time", time.getEpochSecond());
        frameJson.put("time_ns", time.getNano());
        JSONArray symbolsArray = new JSONArray();
        for (TelemetrySymbol s : symbols) {
            symbolsArray.put(s.toJson());
        }
        if (symbolsArray.length() > 0) {
            frameJson.put("symbols", symbolsArray);
        }
        return frameJson;
    }

    @Override
    public String toString() {
        return "TelemetryFrame{" +
                "symbols=" + symbols +
                ", time=" + time +
                '}';
    }

    /**
     * Return the symbols associated with this frame.
     */
    public Set<TelemetrySymbol> getSymbols() {
        return symbols;
    }

    public void addSymbol(TelemetrySymbol symbol) {
        // This works on the principal that equals() is implemented properly
        symbols.add(symbol);
    }

}
